// CodeTrace.h
//

#ifndef CODETRACE_H
#define CODETRACE_H

#ifdef TRACE_ON

#ifdef _WIN32
#include <windows.h>
#endif

#include <fstream>
#include <iostream>
#include <typeinfo>
#include <string>
#include <sstream>
#include <cassert>

class CodeTrace {
	CodeTrace () {};
	void operator= (CodeTrace&);
	CodeTrace (const CodeTrace&);	
public:
	template <typename T1> 
		static void print (T1 par1, std::string file, int line)	{
		std::stringstream a;
		a << getCount () << ": " << file << " (" << line << "): " << par1;
		
		serialize () << a.str () << std::endl;
		std::cout << a.str () << std::endl;
		
		#ifdef _WIN32
		OutputDebugString (a.str ().c_str ());
		#endif
	}
	template <typename T1> 
		static void trace (T1 par1, std::string name,
		std::string file, int line) {
		std::stringstream a;
		a << " (" << typeid (par1).name () << ") " << name << " = " << par1;

		print (a.str (), file, line);
	}
private:
	static long getCount () {
		static long count = -1;
		count++;
		return count;
	}
	static std::ofstream& CodeTrace::serialize () { // Stream creation on use...
		static std::ofstream __TRACEFILE__ ("CodeTrace.log");
		assert (__TRACEFILE__);
	return __TRACEFILE__;
	}
};
	#define PRINT(a) CodeTrace::print (a, __FILE__, __LINE__);
	#define TRACE(a) CodeTrace::trace (a, #a, __FILE__, __LINE__);
#else
	#define PRINT(a) ((void) 0);
	#define TRACE(a) ((void) 0);
#endif	// TRACE_ON

#endif	// CODETRACE_H
